# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'GetAuthBackendConfigResult',
    'AwaitableGetAuthBackendConfigResult',
    'get_auth_backend_config',
    'get_auth_backend_config_output',
]

@pulumi.output_type
class GetAuthBackendConfigResult:
    """
    A collection of values returned by getAuthBackendConfig.
    """
    def __init__(__self__, backend=None, disable_iss_validation=None, disable_local_ca_jwt=None, id=None, issuer=None, kubernetes_ca_cert=None, kubernetes_host=None, namespace=None, pem_keys=None):
        if backend and not isinstance(backend, str):
            raise TypeError("Expected argument 'backend' to be a str")
        pulumi.set(__self__, "backend", backend)
        if disable_iss_validation and not isinstance(disable_iss_validation, bool):
            raise TypeError("Expected argument 'disable_iss_validation' to be a bool")
        pulumi.set(__self__, "disable_iss_validation", disable_iss_validation)
        if disable_local_ca_jwt and not isinstance(disable_local_ca_jwt, bool):
            raise TypeError("Expected argument 'disable_local_ca_jwt' to be a bool")
        pulumi.set(__self__, "disable_local_ca_jwt", disable_local_ca_jwt)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if issuer and not isinstance(issuer, str):
            raise TypeError("Expected argument 'issuer' to be a str")
        pulumi.set(__self__, "issuer", issuer)
        if kubernetes_ca_cert and not isinstance(kubernetes_ca_cert, str):
            raise TypeError("Expected argument 'kubernetes_ca_cert' to be a str")
        pulumi.set(__self__, "kubernetes_ca_cert", kubernetes_ca_cert)
        if kubernetes_host and not isinstance(kubernetes_host, str):
            raise TypeError("Expected argument 'kubernetes_host' to be a str")
        pulumi.set(__self__, "kubernetes_host", kubernetes_host)
        if namespace and not isinstance(namespace, str):
            raise TypeError("Expected argument 'namespace' to be a str")
        pulumi.set(__self__, "namespace", namespace)
        if pem_keys and not isinstance(pem_keys, list):
            raise TypeError("Expected argument 'pem_keys' to be a list")
        pulumi.set(__self__, "pem_keys", pem_keys)

    @property
    @pulumi.getter
    def backend(self) -> Optional[str]:
        return pulumi.get(self, "backend")

    @property
    @pulumi.getter(name="disableIssValidation")
    def disable_iss_validation(self) -> bool:
        return pulumi.get(self, "disable_iss_validation")

    @property
    @pulumi.getter(name="disableLocalCaJwt")
    def disable_local_ca_jwt(self) -> bool:
        return pulumi.get(self, "disable_local_ca_jwt")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def issuer(self) -> str:
        """
        Optional JWT issuer. If no issuer is specified, `kubernetes.io/serviceaccount` will be used as the default issuer.
        """
        return pulumi.get(self, "issuer")

    @property
    @pulumi.getter(name="kubernetesCaCert")
    def kubernetes_ca_cert(self) -> str:
        """
        PEM encoded CA cert for use by the TLS client used to talk with the Kubernetes API.
        """
        return pulumi.get(self, "kubernetes_ca_cert")

    @property
    @pulumi.getter(name="kubernetesHost")
    def kubernetes_host(self) -> str:
        """
        Host must be a host string, a host:port pair, or a URL to the base of the Kubernetes API server.
        """
        return pulumi.get(self, "kubernetes_host")

    @property
    @pulumi.getter
    def namespace(self) -> Optional[str]:
        return pulumi.get(self, "namespace")

    @property
    @pulumi.getter(name="pemKeys")
    def pem_keys(self) -> Sequence[str]:
        """
        Optional list of PEM-formatted public keys or certificates used to verify the signatures of Kubernetes service account JWTs. If a certificate is given, its public key will be extracted. Not every installation of Kubernetes exposes these keys.
        """
        return pulumi.get(self, "pem_keys")


class AwaitableGetAuthBackendConfigResult(GetAuthBackendConfigResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAuthBackendConfigResult(
            backend=self.backend,
            disable_iss_validation=self.disable_iss_validation,
            disable_local_ca_jwt=self.disable_local_ca_jwt,
            id=self.id,
            issuer=self.issuer,
            kubernetes_ca_cert=self.kubernetes_ca_cert,
            kubernetes_host=self.kubernetes_host,
            namespace=self.namespace,
            pem_keys=self.pem_keys)


def get_auth_backend_config(backend: Optional[str] = None,
                            disable_iss_validation: Optional[bool] = None,
                            disable_local_ca_jwt: Optional[bool] = None,
                            issuer: Optional[str] = None,
                            kubernetes_ca_cert: Optional[str] = None,
                            kubernetes_host: Optional[str] = None,
                            namespace: Optional[str] = None,
                            pem_keys: Optional[Sequence[str]] = None,
                            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetAuthBackendConfigResult:
    """
    Reads the Role of an Kubernetes from a Vault server. See the [Vault
    documentation](https://www.vaultproject.io/api-docs/auth/kubernetes#read-config) for more
    information.


    :param str backend: The unique name for the Kubernetes backend the config to
           retrieve Role attributes for resides in. Defaults to "kubernetes".
    :param str issuer: Optional JWT issuer. If no issuer is specified, `kubernetes.io/serviceaccount` will be used as the default issuer.
    :param str kubernetes_ca_cert: PEM encoded CA cert for use by the TLS client used to talk with the Kubernetes API.
    :param str kubernetes_host: Host must be a host string, a host:port pair, or a URL to the base of the Kubernetes API server.
    :param str namespace: The namespace of the target resource.
           The value should not contain leading or trailing forward slashes.
           The `namespace` is always relative to the provider's configured namespace.
           *Available only for Vault Enterprise*.
    :param Sequence[str] pem_keys: Optional list of PEM-formatted public keys or certificates used to verify the signatures of Kubernetes service account JWTs. If a certificate is given, its public key will be extracted. Not every installation of Kubernetes exposes these keys.
    """
    __args__ = dict()
    __args__['backend'] = backend
    __args__['disableIssValidation'] = disable_iss_validation
    __args__['disableLocalCaJwt'] = disable_local_ca_jwt
    __args__['issuer'] = issuer
    __args__['kubernetesCaCert'] = kubernetes_ca_cert
    __args__['kubernetesHost'] = kubernetes_host
    __args__['namespace'] = namespace
    __args__['pemKeys'] = pem_keys
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('vault:kubernetes/getAuthBackendConfig:getAuthBackendConfig', __args__, opts=opts, typ=GetAuthBackendConfigResult).value

    return AwaitableGetAuthBackendConfigResult(
        backend=pulumi.get(__ret__, 'backend'),
        disable_iss_validation=pulumi.get(__ret__, 'disable_iss_validation'),
        disable_local_ca_jwt=pulumi.get(__ret__, 'disable_local_ca_jwt'),
        id=pulumi.get(__ret__, 'id'),
        issuer=pulumi.get(__ret__, 'issuer'),
        kubernetes_ca_cert=pulumi.get(__ret__, 'kubernetes_ca_cert'),
        kubernetes_host=pulumi.get(__ret__, 'kubernetes_host'),
        namespace=pulumi.get(__ret__, 'namespace'),
        pem_keys=pulumi.get(__ret__, 'pem_keys'))


@_utilities.lift_output_func(get_auth_backend_config)
def get_auth_backend_config_output(backend: Optional[pulumi.Input[Optional[str]]] = None,
                                   disable_iss_validation: Optional[pulumi.Input[Optional[bool]]] = None,
                                   disable_local_ca_jwt: Optional[pulumi.Input[Optional[bool]]] = None,
                                   issuer: Optional[pulumi.Input[Optional[str]]] = None,
                                   kubernetes_ca_cert: Optional[pulumi.Input[Optional[str]]] = None,
                                   kubernetes_host: Optional[pulumi.Input[Optional[str]]] = None,
                                   namespace: Optional[pulumi.Input[Optional[str]]] = None,
                                   pem_keys: Optional[pulumi.Input[Optional[Sequence[str]]]] = None,
                                   opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetAuthBackendConfigResult]:
    """
    Reads the Role of an Kubernetes from a Vault server. See the [Vault
    documentation](https://www.vaultproject.io/api-docs/auth/kubernetes#read-config) for more
    information.


    :param str backend: The unique name for the Kubernetes backend the config to
           retrieve Role attributes for resides in. Defaults to "kubernetes".
    :param str issuer: Optional JWT issuer. If no issuer is specified, `kubernetes.io/serviceaccount` will be used as the default issuer.
    :param str kubernetes_ca_cert: PEM encoded CA cert for use by the TLS client used to talk with the Kubernetes API.
    :param str kubernetes_host: Host must be a host string, a host:port pair, or a URL to the base of the Kubernetes API server.
    :param str namespace: The namespace of the target resource.
           The value should not contain leading or trailing forward slashes.
           The `namespace` is always relative to the provider's configured namespace.
           *Available only for Vault Enterprise*.
    :param Sequence[str] pem_keys: Optional list of PEM-formatted public keys or certificates used to verify the signatures of Kubernetes service account JWTs. If a certificate is given, its public key will be extracted. Not every installation of Kubernetes exposes these keys.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['SecretBackendRoleArgs', 'SecretBackendRole']

@pulumi.input_type
class SecretBackendRoleArgs:
    def __init__(__self__, *,
                 allowed_kubernetes_namespaces: pulumi.Input[Sequence[pulumi.Input[str]]],
                 backend: pulumi.Input[str],
                 extra_annotations: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 extra_labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 generated_role_rules: Optional[pulumi.Input[str]] = None,
                 kubernetes_role_name: Optional[pulumi.Input[str]] = None,
                 kubernetes_role_type: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 name_template: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 service_account_name: Optional[pulumi.Input[str]] = None,
                 token_default_ttl: Optional[pulumi.Input[int]] = None,
                 token_max_ttl: Optional[pulumi.Input[int]] = None):
        """
        The set of arguments for constructing a SecretBackendRole resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] allowed_kubernetes_namespaces: The list of Kubernetes namespaces this role 
               can generate credentials for. If set to `*` all namespaces are allowed.
        :param pulumi.Input[str] backend: The path of the Kubernetes Secrets Engine backend mount to create
               the role in.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] extra_annotations: Additional annotations to apply to all generated 
               Kubernetes objects.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] extra_labels: Additional labels to apply to all generated Kubernetes 
               objects.
               
               This resource also directly accepts all Mount fields.
        :param pulumi.Input[str] generated_role_rules: The Role or ClusterRole rules to use when generating 
               a role. Accepts either JSON or YAML formatted rules. Mutually exclusive with `service_account_name`
               and `kubernetes_role_name`. If set, the entire chain of Kubernetes objects will be generated
               when credentials are requested.
        :param pulumi.Input[str] kubernetes_role_name: The pre-existing Role or ClusterRole to bind a 
               generated service account to. Mutually exclusive with `service_account_name` and
               `generated_role_rules`. If set, Kubernetes token, service account, and role
               binding objects will be created when credentials are requested.
        :param pulumi.Input[str] kubernetes_role_type: Specifies whether the Kubernetes role is a Role or 
               ClusterRole.
        :param pulumi.Input[str] name: The name of the role.
        :param pulumi.Input[str] name_template: The name template to use when generating service accounts, 
               roles and role bindings. If unset, a default template is used.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[str] service_account_name: The pre-existing service account to generate tokens for.
               Mutually exclusive with `kubernetes_role_name` and `generated_role_rules`. If set, only a
               Kubernetes token will be created when credentials are requested.
        :param pulumi.Input[int] token_default_ttl: The default TTL for generated Kubernetes tokens in seconds.
        :param pulumi.Input[int] token_max_ttl: The maximum TTL for generated Kubernetes tokens in seconds.
        """
        pulumi.set(__self__, "allowed_kubernetes_namespaces", allowed_kubernetes_namespaces)
        pulumi.set(__self__, "backend", backend)
        if extra_annotations is not None:
            pulumi.set(__self__, "extra_annotations", extra_annotations)
        if extra_labels is not None:
            pulumi.set(__self__, "extra_labels", extra_labels)
        if generated_role_rules is not None:
            pulumi.set(__self__, "generated_role_rules", generated_role_rules)
        if kubernetes_role_name is not None:
            pulumi.set(__self__, "kubernetes_role_name", kubernetes_role_name)
        if kubernetes_role_type is not None:
            pulumi.set(__self__, "kubernetes_role_type", kubernetes_role_type)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if name_template is not None:
            pulumi.set(__self__, "name_template", name_template)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if service_account_name is not None:
            pulumi.set(__self__, "service_account_name", service_account_name)
        if token_default_ttl is not None:
            pulumi.set(__self__, "token_default_ttl", token_default_ttl)
        if token_max_ttl is not None:
            pulumi.set(__self__, "token_max_ttl", token_max_ttl)

    @property
    @pulumi.getter(name="allowedKubernetesNamespaces")
    def allowed_kubernetes_namespaces(self) -> pulumi.Input[Sequence[pulumi.Input[str]]]:
        """
        The list of Kubernetes namespaces this role 
        can generate credentials for. If set to `*` all namespaces are allowed.
        """
        return pulumi.get(self, "allowed_kubernetes_namespaces")

    @allowed_kubernetes_namespaces.setter
    def allowed_kubernetes_namespaces(self, value: pulumi.Input[Sequence[pulumi.Input[str]]]):
        pulumi.set(self, "allowed_kubernetes_namespaces", value)

    @property
    @pulumi.getter
    def backend(self) -> pulumi.Input[str]:
        """
        The path of the Kubernetes Secrets Engine backend mount to create
        the role in.
        """
        return pulumi.get(self, "backend")

    @backend.setter
    def backend(self, value: pulumi.Input[str]):
        pulumi.set(self, "backend", value)

    @property
    @pulumi.getter(name="extraAnnotations")
    def extra_annotations(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Additional annotations to apply to all generated 
        Kubernetes objects.
        """
        return pulumi.get(self, "extra_annotations")

    @extra_annotations.setter
    def extra_annotations(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "extra_annotations", value)

    @property
    @pulumi.getter(name="extraLabels")
    def extra_labels(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Additional labels to apply to all generated Kubernetes 
        objects.

        This resource also directly accepts all Mount fields.
        """
        return pulumi.get(self, "extra_labels")

    @extra_labels.setter
    def extra_labels(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "extra_labels", value)

    @property
    @pulumi.getter(name="generatedRoleRules")
    def generated_role_rules(self) -> Optional[pulumi.Input[str]]:
        """
        The Role or ClusterRole rules to use when generating 
        a role. Accepts either JSON or YAML formatted rules. Mutually exclusive with `service_account_name`
        and `kubernetes_role_name`. If set, the entire chain of Kubernetes objects will be generated
        when credentials are requested.
        """
        return pulumi.get(self, "generated_role_rules")

    @generated_role_rules.setter
    def generated_role_rules(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "generated_role_rules", value)

    @property
    @pulumi.getter(name="kubernetesRoleName")
    def kubernetes_role_name(self) -> Optional[pulumi.Input[str]]:
        """
        The pre-existing Role or ClusterRole to bind a 
        generated service account to. Mutually exclusive with `service_account_name` and
        `generated_role_rules`. If set, Kubernetes token, service account, and role
        binding objects will be created when credentials are requested.
        """
        return pulumi.get(self, "kubernetes_role_name")

    @kubernetes_role_name.setter
    def kubernetes_role_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "kubernetes_role_name", value)

    @property
    @pulumi.getter(name="kubernetesRoleType")
    def kubernetes_role_type(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies whether the Kubernetes role is a Role or 
        ClusterRole.
        """
        return pulumi.get(self, "kubernetes_role_type")

    @kubernetes_role_type.setter
    def kubernetes_role_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "kubernetes_role_type", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the role.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="nameTemplate")
    def name_template(self) -> Optional[pulumi.Input[str]]:
        """
        The name template to use when generating service accounts, 
        roles and role bindings. If unset, a default template is used.
        """
        return pulumi.get(self, "name_template")

    @name_template.setter
    def name_template(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name_template", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        The namespace to provision the resource in.
        The value should not contain leading or trailing forward slashes.
        The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
        *Available only for Vault Enterprise*.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter(name="serviceAccountName")
    def service_account_name(self) -> Optional[pulumi.Input[str]]:
        """
        The pre-existing service account to generate tokens for.
        Mutually exclusive with `kubernetes_role_name` and `generated_role_rules`. If set, only a
        Kubernetes token will be created when credentials are requested.
        """
        return pulumi.get(self, "service_account_name")

    @service_account_name.setter
    def service_account_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service_account_name", value)

    @property
    @pulumi.getter(name="tokenDefaultTtl")
    def token_default_ttl(self) -> Optional[pulumi.Input[int]]:
        """
        The default TTL for generated Kubernetes tokens in seconds.
        """
        return pulumi.get(self, "token_default_ttl")

    @token_default_ttl.setter
    def token_default_ttl(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "token_default_ttl", value)

    @property
    @pulumi.getter(name="tokenMaxTtl")
    def token_max_ttl(self) -> Optional[pulumi.Input[int]]:
        """
        The maximum TTL for generated Kubernetes tokens in seconds.
        """
        return pulumi.get(self, "token_max_ttl")

    @token_max_ttl.setter
    def token_max_ttl(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "token_max_ttl", value)


@pulumi.input_type
class _SecretBackendRoleState:
    def __init__(__self__, *,
                 allowed_kubernetes_namespaces: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 backend: Optional[pulumi.Input[str]] = None,
                 extra_annotations: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 extra_labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 generated_role_rules: Optional[pulumi.Input[str]] = None,
                 kubernetes_role_name: Optional[pulumi.Input[str]] = None,
                 kubernetes_role_type: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 name_template: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 service_account_name: Optional[pulumi.Input[str]] = None,
                 token_default_ttl: Optional[pulumi.Input[int]] = None,
                 token_max_ttl: Optional[pulumi.Input[int]] = None):
        """
        Input properties used for looking up and filtering SecretBackendRole resources.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] allowed_kubernetes_namespaces: The list of Kubernetes namespaces this role 
               can generate credentials for. If set to `*` all namespaces are allowed.
        :param pulumi.Input[str] backend: The path of the Kubernetes Secrets Engine backend mount to create
               the role in.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] extra_annotations: Additional annotations to apply to all generated 
               Kubernetes objects.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] extra_labels: Additional labels to apply to all generated Kubernetes 
               objects.
               
               This resource also directly accepts all Mount fields.
        :param pulumi.Input[str] generated_role_rules: The Role or ClusterRole rules to use when generating 
               a role. Accepts either JSON or YAML formatted rules. Mutually exclusive with `service_account_name`
               and `kubernetes_role_name`. If set, the entire chain of Kubernetes objects will be generated
               when credentials are requested.
        :param pulumi.Input[str] kubernetes_role_name: The pre-existing Role or ClusterRole to bind a 
               generated service account to. Mutually exclusive with `service_account_name` and
               `generated_role_rules`. If set, Kubernetes token, service account, and role
               binding objects will be created when credentials are requested.
        :param pulumi.Input[str] kubernetes_role_type: Specifies whether the Kubernetes role is a Role or 
               ClusterRole.
        :param pulumi.Input[str] name: The name of the role.
        :param pulumi.Input[str] name_template: The name template to use when generating service accounts, 
               roles and role bindings. If unset, a default template is used.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[str] service_account_name: The pre-existing service account to generate tokens for.
               Mutually exclusive with `kubernetes_role_name` and `generated_role_rules`. If set, only a
               Kubernetes token will be created when credentials are requested.
        :param pulumi.Input[int] token_default_ttl: The default TTL for generated Kubernetes tokens in seconds.
        :param pulumi.Input[int] token_max_ttl: The maximum TTL for generated Kubernetes tokens in seconds.
        """
        if allowed_kubernetes_namespaces is not None:
            pulumi.set(__self__, "allowed_kubernetes_namespaces", allowed_kubernetes_namespaces)
        if backend is not None:
            pulumi.set(__self__, "backend", backend)
        if extra_annotations is not None:
            pulumi.set(__self__, "extra_annotations", extra_annotations)
        if extra_labels is not None:
            pulumi.set(__self__, "extra_labels", extra_labels)
        if generated_role_rules is not None:
            pulumi.set(__self__, "generated_role_rules", generated_role_rules)
        if kubernetes_role_name is not None:
            pulumi.set(__self__, "kubernetes_role_name", kubernetes_role_name)
        if kubernetes_role_type is not None:
            pulumi.set(__self__, "kubernetes_role_type", kubernetes_role_type)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if name_template is not None:
            pulumi.set(__self__, "name_template", name_template)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if service_account_name is not None:
            pulumi.set(__self__, "service_account_name", service_account_name)
        if token_default_ttl is not None:
            pulumi.set(__self__, "token_default_ttl", token_default_ttl)
        if token_max_ttl is not None:
            pulumi.set(__self__, "token_max_ttl", token_max_ttl)

    @property
    @pulumi.getter(name="allowedKubernetesNamespaces")
    def allowed_kubernetes_namespaces(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The list of Kubernetes namespaces this role 
        can generate credentials for. If set to `*` all namespaces are allowed.
        """
        return pulumi.get(self, "allowed_kubernetes_namespaces")

    @allowed_kubernetes_namespaces.setter
    def allowed_kubernetes_namespaces(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "allowed_kubernetes_namespaces", value)

    @property
    @pulumi.getter
    def backend(self) -> Optional[pulumi.Input[str]]:
        """
        The path of the Kubernetes Secrets Engine backend mount to create
        the role in.
        """
        return pulumi.get(self, "backend")

    @backend.setter
    def backend(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "backend", value)

    @property
    @pulumi.getter(name="extraAnnotations")
    def extra_annotations(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Additional annotations to apply to all generated 
        Kubernetes objects.
        """
        return pulumi.get(self, "extra_annotations")

    @extra_annotations.setter
    def extra_annotations(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "extra_annotations", value)

    @property
    @pulumi.getter(name="extraLabels")
    def extra_labels(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Additional labels to apply to all generated Kubernetes 
        objects.

        This resource also directly accepts all Mount fields.
        """
        return pulumi.get(self, "extra_labels")

    @extra_labels.setter
    def extra_labels(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "extra_labels", value)

    @property
    @pulumi.getter(name="generatedRoleRules")
    def generated_role_rules(self) -> Optional[pulumi.Input[str]]:
        """
        The Role or ClusterRole rules to use when generating 
        a role. Accepts either JSON or YAML formatted rules. Mutually exclusive with `service_account_name`
        and `kubernetes_role_name`. If set, the entire chain of Kubernetes objects will be generated
        when credentials are requested.
        """
        return pulumi.get(self, "generated_role_rules")

    @generated_role_rules.setter
    def generated_role_rules(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "generated_role_rules", value)

    @property
    @pulumi.getter(name="kubernetesRoleName")
    def kubernetes_role_name(self) -> Optional[pulumi.Input[str]]:
        """
        The pre-existing Role or ClusterRole to bind a 
        generated service account to. Mutually exclusive with `service_account_name` and
        `generated_role_rules`. If set, Kubernetes token, service account, and role
        binding objects will be created when credentials are requested.
        """
        return pulumi.get(self, "kubernetes_role_name")

    @kubernetes_role_name.setter
    def kubernetes_role_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "kubernetes_role_name", value)

    @property
    @pulumi.getter(name="kubernetesRoleType")
    def kubernetes_role_type(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies whether the Kubernetes role is a Role or 
        ClusterRole.
        """
        return pulumi.get(self, "kubernetes_role_type")

    @kubernetes_role_type.setter
    def kubernetes_role_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "kubernetes_role_type", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the role.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="nameTemplate")
    def name_template(self) -> Optional[pulumi.Input[str]]:
        """
        The name template to use when generating service accounts, 
        roles and role bindings. If unset, a default template is used.
        """
        return pulumi.get(self, "name_template")

    @name_template.setter
    def name_template(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name_template", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        The namespace to provision the resource in.
        The value should not contain leading or trailing forward slashes.
        The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
        *Available only for Vault Enterprise*.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter(name="serviceAccountName")
    def service_account_name(self) -> Optional[pulumi.Input[str]]:
        """
        The pre-existing service account to generate tokens for.
        Mutually exclusive with `kubernetes_role_name` and `generated_role_rules`. If set, only a
        Kubernetes token will be created when credentials are requested.
        """
        return pulumi.get(self, "service_account_name")

    @service_account_name.setter
    def service_account_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service_account_name", value)

    @property
    @pulumi.getter(name="tokenDefaultTtl")
    def token_default_ttl(self) -> Optional[pulumi.Input[int]]:
        """
        The default TTL for generated Kubernetes tokens in seconds.
        """
        return pulumi.get(self, "token_default_ttl")

    @token_default_ttl.setter
    def token_default_ttl(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "token_default_ttl", value)

    @property
    @pulumi.getter(name="tokenMaxTtl")
    def token_max_ttl(self) -> Optional[pulumi.Input[int]]:
        """
        The maximum TTL for generated Kubernetes tokens in seconds.
        """
        return pulumi.get(self, "token_max_ttl")

    @token_max_ttl.setter
    def token_max_ttl(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "token_max_ttl", value)


class SecretBackendRole(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 allowed_kubernetes_namespaces: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 backend: Optional[pulumi.Input[str]] = None,
                 extra_annotations: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 extra_labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 generated_role_rules: Optional[pulumi.Input[str]] = None,
                 kubernetes_role_name: Optional[pulumi.Input[str]] = None,
                 kubernetes_role_type: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 name_template: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 service_account_name: Optional[pulumi.Input[str]] = None,
                 token_default_ttl: Optional[pulumi.Input[int]] = None,
                 token_max_ttl: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        """
        ## Example Usage

        Example using `service_account_name` mode:

        ```python
        import pulumi
        import pulumi_vault as vault

        config = vault.kubernetes.SecretBackend("config",
            path="kubernetes",
            description="kubernetes secrets engine description",
            kubernetes_host="https://127.0.0.1:61233",
            kubernetes_ca_cert=(lambda path: open(path).read())("/path/to/cert"),
            service_account_jwt=(lambda path: open(path).read())("/path/to/token"),
            disable_local_ca_jwt=False)
        sa_example = vault.kubernetes.SecretBackendRole("sa-example",
            backend=config.path,
            allowed_kubernetes_namespaces=["*"],
            token_max_ttl=43200,
            token_default_ttl=21600,
            service_account_name="test-service-account-with-generated-token",
            extra_labels={
                "id": "abc123",
                "name": "some_name",
            },
            extra_annotations={
                "env": "development",
                "location": "earth",
            })
        ```

        Example using `kubernetes_role_name` mode:

        ```python
        import pulumi
        import pulumi_vault as vault

        config = vault.kubernetes.SecretBackend("config",
            path="kubernetes",
            description="kubernetes secrets engine description",
            kubernetes_host="https://127.0.0.1:61233",
            kubernetes_ca_cert=(lambda path: open(path).read())("/path/to/cert"),
            service_account_jwt=(lambda path: open(path).read())("/path/to/token"),
            disable_local_ca_jwt=False)
        name_example = vault.kubernetes.SecretBackendRole("name-example",
            backend=config.path,
            allowed_kubernetes_namespaces=["*"],
            token_max_ttl=43200,
            token_default_ttl=21600,
            kubernetes_role_name="vault-k8s-secrets-role",
            extra_labels={
                "id": "abc123",
                "name": "some_name",
            },
            extra_annotations={
                "env": "development",
                "location": "earth",
            })
        ```

        Example using `generated_role_rules` mode:

        ```python
        import pulumi
        import pulumi_vault as vault

        config = vault.kubernetes.SecretBackend("config",
            path="kubernetes",
            description="kubernetes secrets engine description",
            kubernetes_host="https://127.0.0.1:61233",
            kubernetes_ca_cert=(lambda path: open(path).read())("/path/to/cert"),
            service_account_jwt=(lambda path: open(path).read())("/path/to/token"),
            disable_local_ca_jwt=False)
        rules_example = vault.kubernetes.SecretBackendRole("rules-example",
            backend=config.path,
            allowed_kubernetes_namespaces=["*"],
            token_max_ttl=43200,
            token_default_ttl=21600,
            kubernetes_role_type="Role",
            generated_role_rules=\"\"\"rules:
        - apiGroups: [""]
          resources: ["pods"]
          verbs: ["list"]
        \"\"\",
            extra_labels={
                "id": "abc123",
                "name": "some_name",
            },
            extra_annotations={
                "env": "development",
                "location": "earth",
            })
        ```

        ## Import

        The Kubernetes secret backend role can be imported using the full path to the role of the form`<backend_path>/roles/<role_name>` e.g.

        ```sh
         $ pulumi import vault:kubernetes/secretBackendRole:SecretBackendRole example kubernetes kubernetes/roles/example-role
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] allowed_kubernetes_namespaces: The list of Kubernetes namespaces this role 
               can generate credentials for. If set to `*` all namespaces are allowed.
        :param pulumi.Input[str] backend: The path of the Kubernetes Secrets Engine backend mount to create
               the role in.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] extra_annotations: Additional annotations to apply to all generated 
               Kubernetes objects.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] extra_labels: Additional labels to apply to all generated Kubernetes 
               objects.
               
               This resource also directly accepts all Mount fields.
        :param pulumi.Input[str] generated_role_rules: The Role or ClusterRole rules to use when generating 
               a role. Accepts either JSON or YAML formatted rules. Mutually exclusive with `service_account_name`
               and `kubernetes_role_name`. If set, the entire chain of Kubernetes objects will be generated
               when credentials are requested.
        :param pulumi.Input[str] kubernetes_role_name: The pre-existing Role or ClusterRole to bind a 
               generated service account to. Mutually exclusive with `service_account_name` and
               `generated_role_rules`. If set, Kubernetes token, service account, and role
               binding objects will be created when credentials are requested.
        :param pulumi.Input[str] kubernetes_role_type: Specifies whether the Kubernetes role is a Role or 
               ClusterRole.
        :param pulumi.Input[str] name: The name of the role.
        :param pulumi.Input[str] name_template: The name template to use when generating service accounts, 
               roles and role bindings. If unset, a default template is used.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[str] service_account_name: The pre-existing service account to generate tokens for.
               Mutually exclusive with `kubernetes_role_name` and `generated_role_rules`. If set, only a
               Kubernetes token will be created when credentials are requested.
        :param pulumi.Input[int] token_default_ttl: The default TTL for generated Kubernetes tokens in seconds.
        :param pulumi.Input[int] token_max_ttl: The maximum TTL for generated Kubernetes tokens in seconds.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: SecretBackendRoleArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        Example using `service_account_name` mode:

        ```python
        import pulumi
        import pulumi_vault as vault

        config = vault.kubernetes.SecretBackend("config",
            path="kubernetes",
            description="kubernetes secrets engine description",
            kubernetes_host="https://127.0.0.1:61233",
            kubernetes_ca_cert=(lambda path: open(path).read())("/path/to/cert"),
            service_account_jwt=(lambda path: open(path).read())("/path/to/token"),
            disable_local_ca_jwt=False)
        sa_example = vault.kubernetes.SecretBackendRole("sa-example",
            backend=config.path,
            allowed_kubernetes_namespaces=["*"],
            token_max_ttl=43200,
            token_default_ttl=21600,
            service_account_name="test-service-account-with-generated-token",
            extra_labels={
                "id": "abc123",
                "name": "some_name",
            },
            extra_annotations={
                "env": "development",
                "location": "earth",
            })
        ```

        Example using `kubernetes_role_name` mode:

        ```python
        import pulumi
        import pulumi_vault as vault

        config = vault.kubernetes.SecretBackend("config",
            path="kubernetes",
            description="kubernetes secrets engine description",
            kubernetes_host="https://127.0.0.1:61233",
            kubernetes_ca_cert=(lambda path: open(path).read())("/path/to/cert"),
            service_account_jwt=(lambda path: open(path).read())("/path/to/token"),
            disable_local_ca_jwt=False)
        name_example = vault.kubernetes.SecretBackendRole("name-example",
            backend=config.path,
            allowed_kubernetes_namespaces=["*"],
            token_max_ttl=43200,
            token_default_ttl=21600,
            kubernetes_role_name="vault-k8s-secrets-role",
            extra_labels={
                "id": "abc123",
                "name": "some_name",
            },
            extra_annotations={
                "env": "development",
                "location": "earth",
            })
        ```

        Example using `generated_role_rules` mode:

        ```python
        import pulumi
        import pulumi_vault as vault

        config = vault.kubernetes.SecretBackend("config",
            path="kubernetes",
            description="kubernetes secrets engine description",
            kubernetes_host="https://127.0.0.1:61233",
            kubernetes_ca_cert=(lambda path: open(path).read())("/path/to/cert"),
            service_account_jwt=(lambda path: open(path).read())("/path/to/token"),
            disable_local_ca_jwt=False)
        rules_example = vault.kubernetes.SecretBackendRole("rules-example",
            backend=config.path,
            allowed_kubernetes_namespaces=["*"],
            token_max_ttl=43200,
            token_default_ttl=21600,
            kubernetes_role_type="Role",
            generated_role_rules=\"\"\"rules:
        - apiGroups: [""]
          resources: ["pods"]
          verbs: ["list"]
        \"\"\",
            extra_labels={
                "id": "abc123",
                "name": "some_name",
            },
            extra_annotations={
                "env": "development",
                "location": "earth",
            })
        ```

        ## Import

        The Kubernetes secret backend role can be imported using the full path to the role of the form`<backend_path>/roles/<role_name>` e.g.

        ```sh
         $ pulumi import vault:kubernetes/secretBackendRole:SecretBackendRole example kubernetes kubernetes/roles/example-role
        ```

        :param str resource_name: The name of the resource.
        :param SecretBackendRoleArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(SecretBackendRoleArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 allowed_kubernetes_namespaces: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 backend: Optional[pulumi.Input[str]] = None,
                 extra_annotations: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 extra_labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 generated_role_rules: Optional[pulumi.Input[str]] = None,
                 kubernetes_role_name: Optional[pulumi.Input[str]] = None,
                 kubernetes_role_type: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 name_template: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 service_account_name: Optional[pulumi.Input[str]] = None,
                 token_default_ttl: Optional[pulumi.Input[int]] = None,
                 token_max_ttl: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = SecretBackendRoleArgs.__new__(SecretBackendRoleArgs)

            if allowed_kubernetes_namespaces is None and not opts.urn:
                raise TypeError("Missing required property 'allowed_kubernetes_namespaces'")
            __props__.__dict__["allowed_kubernetes_namespaces"] = allowed_kubernetes_namespaces
            if backend is None and not opts.urn:
                raise TypeError("Missing required property 'backend'")
            __props__.__dict__["backend"] = backend
            __props__.__dict__["extra_annotations"] = extra_annotations
            __props__.__dict__["extra_labels"] = extra_labels
            __props__.__dict__["generated_role_rules"] = generated_role_rules
            __props__.__dict__["kubernetes_role_name"] = kubernetes_role_name
            __props__.__dict__["kubernetes_role_type"] = kubernetes_role_type
            __props__.__dict__["name"] = name
            __props__.__dict__["name_template"] = name_template
            __props__.__dict__["namespace"] = namespace
            __props__.__dict__["service_account_name"] = service_account_name
            __props__.__dict__["token_default_ttl"] = token_default_ttl
            __props__.__dict__["token_max_ttl"] = token_max_ttl
        super(SecretBackendRole, __self__).__init__(
            'vault:kubernetes/secretBackendRole:SecretBackendRole',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            allowed_kubernetes_namespaces: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            backend: Optional[pulumi.Input[str]] = None,
            extra_annotations: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            extra_labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            generated_role_rules: Optional[pulumi.Input[str]] = None,
            kubernetes_role_name: Optional[pulumi.Input[str]] = None,
            kubernetes_role_type: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            name_template: Optional[pulumi.Input[str]] = None,
            namespace: Optional[pulumi.Input[str]] = None,
            service_account_name: Optional[pulumi.Input[str]] = None,
            token_default_ttl: Optional[pulumi.Input[int]] = None,
            token_max_ttl: Optional[pulumi.Input[int]] = None) -> 'SecretBackendRole':
        """
        Get an existing SecretBackendRole resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] allowed_kubernetes_namespaces: The list of Kubernetes namespaces this role 
               can generate credentials for. If set to `*` all namespaces are allowed.
        :param pulumi.Input[str] backend: The path of the Kubernetes Secrets Engine backend mount to create
               the role in.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] extra_annotations: Additional annotations to apply to all generated 
               Kubernetes objects.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] extra_labels: Additional labels to apply to all generated Kubernetes 
               objects.
               
               This resource also directly accepts all Mount fields.
        :param pulumi.Input[str] generated_role_rules: The Role or ClusterRole rules to use when generating 
               a role. Accepts either JSON or YAML formatted rules. Mutually exclusive with `service_account_name`
               and `kubernetes_role_name`. If set, the entire chain of Kubernetes objects will be generated
               when credentials are requested.
        :param pulumi.Input[str] kubernetes_role_name: The pre-existing Role or ClusterRole to bind a 
               generated service account to. Mutually exclusive with `service_account_name` and
               `generated_role_rules`. If set, Kubernetes token, service account, and role
               binding objects will be created when credentials are requested.
        :param pulumi.Input[str] kubernetes_role_type: Specifies whether the Kubernetes role is a Role or 
               ClusterRole.
        :param pulumi.Input[str] name: The name of the role.
        :param pulumi.Input[str] name_template: The name template to use when generating service accounts, 
               roles and role bindings. If unset, a default template is used.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[str] service_account_name: The pre-existing service account to generate tokens for.
               Mutually exclusive with `kubernetes_role_name` and `generated_role_rules`. If set, only a
               Kubernetes token will be created when credentials are requested.
        :param pulumi.Input[int] token_default_ttl: The default TTL for generated Kubernetes tokens in seconds.
        :param pulumi.Input[int] token_max_ttl: The maximum TTL for generated Kubernetes tokens in seconds.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _SecretBackendRoleState.__new__(_SecretBackendRoleState)

        __props__.__dict__["allowed_kubernetes_namespaces"] = allowed_kubernetes_namespaces
        __props__.__dict__["backend"] = backend
        __props__.__dict__["extra_annotations"] = extra_annotations
        __props__.__dict__["extra_labels"] = extra_labels
        __props__.__dict__["generated_role_rules"] = generated_role_rules
        __props__.__dict__["kubernetes_role_name"] = kubernetes_role_name
        __props__.__dict__["kubernetes_role_type"] = kubernetes_role_type
        __props__.__dict__["name"] = name
        __props__.__dict__["name_template"] = name_template
        __props__.__dict__["namespace"] = namespace
        __props__.__dict__["service_account_name"] = service_account_name
        __props__.__dict__["token_default_ttl"] = token_default_ttl
        __props__.__dict__["token_max_ttl"] = token_max_ttl
        return SecretBackendRole(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="allowedKubernetesNamespaces")
    def allowed_kubernetes_namespaces(self) -> pulumi.Output[Sequence[str]]:
        """
        The list of Kubernetes namespaces this role 
        can generate credentials for. If set to `*` all namespaces are allowed.
        """
        return pulumi.get(self, "allowed_kubernetes_namespaces")

    @property
    @pulumi.getter
    def backend(self) -> pulumi.Output[str]:
        """
        The path of the Kubernetes Secrets Engine backend mount to create
        the role in.
        """
        return pulumi.get(self, "backend")

    @property
    @pulumi.getter(name="extraAnnotations")
    def extra_annotations(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        Additional annotations to apply to all generated 
        Kubernetes objects.
        """
        return pulumi.get(self, "extra_annotations")

    @property
    @pulumi.getter(name="extraLabels")
    def extra_labels(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        Additional labels to apply to all generated Kubernetes 
        objects.

        This resource also directly accepts all Mount fields.
        """
        return pulumi.get(self, "extra_labels")

    @property
    @pulumi.getter(name="generatedRoleRules")
    def generated_role_rules(self) -> pulumi.Output[Optional[str]]:
        """
        The Role or ClusterRole rules to use when generating 
        a role. Accepts either JSON or YAML formatted rules. Mutually exclusive with `service_account_name`
        and `kubernetes_role_name`. If set, the entire chain of Kubernetes objects will be generated
        when credentials are requested.
        """
        return pulumi.get(self, "generated_role_rules")

    @property
    @pulumi.getter(name="kubernetesRoleName")
    def kubernetes_role_name(self) -> pulumi.Output[Optional[str]]:
        """
        The pre-existing Role or ClusterRole to bind a 
        generated service account to. Mutually exclusive with `service_account_name` and
        `generated_role_rules`. If set, Kubernetes token, service account, and role
        binding objects will be created when credentials are requested.
        """
        return pulumi.get(self, "kubernetes_role_name")

    @property
    @pulumi.getter(name="kubernetesRoleType")
    def kubernetes_role_type(self) -> pulumi.Output[Optional[str]]:
        """
        Specifies whether the Kubernetes role is a Role or 
        ClusterRole.
        """
        return pulumi.get(self, "kubernetes_role_type")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        The name of the role.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="nameTemplate")
    def name_template(self) -> pulumi.Output[Optional[str]]:
        """
        The name template to use when generating service accounts, 
        roles and role bindings. If unset, a default template is used.
        """
        return pulumi.get(self, "name_template")

    @property
    @pulumi.getter
    def namespace(self) -> pulumi.Output[Optional[str]]:
        """
        The namespace to provision the resource in.
        The value should not contain leading or trailing forward slashes.
        The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
        *Available only for Vault Enterprise*.
        """
        return pulumi.get(self, "namespace")

    @property
    @pulumi.getter(name="serviceAccountName")
    def service_account_name(self) -> pulumi.Output[Optional[str]]:
        """
        The pre-existing service account to generate tokens for.
        Mutually exclusive with `kubernetes_role_name` and `generated_role_rules`. If set, only a
        Kubernetes token will be created when credentials are requested.
        """
        return pulumi.get(self, "service_account_name")

    @property
    @pulumi.getter(name="tokenDefaultTtl")
    def token_default_ttl(self) -> pulumi.Output[Optional[int]]:
        """
        The default TTL for generated Kubernetes tokens in seconds.
        """
        return pulumi.get(self, "token_default_ttl")

    @property
    @pulumi.getter(name="tokenMaxTtl")
    def token_max_ttl(self) -> pulumi.Output[Optional[int]]:
        """
        The maximum TTL for generated Kubernetes tokens in seconds.
        """
        return pulumi.get(self, "token_max_ttl")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['SecretBackendConfigUrlsArgs', 'SecretBackendConfigUrls']

@pulumi.input_type
class SecretBackendConfigUrlsArgs:
    def __init__(__self__, *,
                 backend: pulumi.Input[str],
                 crl_distribution_points: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 issuing_certificates: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 ocsp_servers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None):
        """
        The set of arguments for constructing a SecretBackendConfigUrls resource.
        :param pulumi.Input[str] backend: The path the PKI secret backend is mounted at, with no leading or trailing `/`s.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] crl_distribution_points: Specifies the URL values for the CRL Distribution Points field.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] issuing_certificates: Specifies the URL values for the Issuing Certificate field.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] ocsp_servers: Specifies the URL values for the OCSP Servers field.
        """
        pulumi.set(__self__, "backend", backend)
        if crl_distribution_points is not None:
            pulumi.set(__self__, "crl_distribution_points", crl_distribution_points)
        if issuing_certificates is not None:
            pulumi.set(__self__, "issuing_certificates", issuing_certificates)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if ocsp_servers is not None:
            pulumi.set(__self__, "ocsp_servers", ocsp_servers)

    @property
    @pulumi.getter
    def backend(self) -> pulumi.Input[str]:
        """
        The path the PKI secret backend is mounted at, with no leading or trailing `/`s.
        """
        return pulumi.get(self, "backend")

    @backend.setter
    def backend(self, value: pulumi.Input[str]):
        pulumi.set(self, "backend", value)

    @property
    @pulumi.getter(name="crlDistributionPoints")
    def crl_distribution_points(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Specifies the URL values for the CRL Distribution Points field.
        """
        return pulumi.get(self, "crl_distribution_points")

    @crl_distribution_points.setter
    def crl_distribution_points(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "crl_distribution_points", value)

    @property
    @pulumi.getter(name="issuingCertificates")
    def issuing_certificates(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Specifies the URL values for the Issuing Certificate field.
        """
        return pulumi.get(self, "issuing_certificates")

    @issuing_certificates.setter
    def issuing_certificates(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "issuing_certificates", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        The namespace to provision the resource in.
        The value should not contain leading or trailing forward slashes.
        The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
        *Available only for Vault Enterprise*.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter(name="ocspServers")
    def ocsp_servers(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Specifies the URL values for the OCSP Servers field.
        """
        return pulumi.get(self, "ocsp_servers")

    @ocsp_servers.setter
    def ocsp_servers(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "ocsp_servers", value)


@pulumi.input_type
class _SecretBackendConfigUrlsState:
    def __init__(__self__, *,
                 backend: Optional[pulumi.Input[str]] = None,
                 crl_distribution_points: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 issuing_certificates: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 ocsp_servers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None):
        """
        Input properties used for looking up and filtering SecretBackendConfigUrls resources.
        :param pulumi.Input[str] backend: The path the PKI secret backend is mounted at, with no leading or trailing `/`s.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] crl_distribution_points: Specifies the URL values for the CRL Distribution Points field.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] issuing_certificates: Specifies the URL values for the Issuing Certificate field.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] ocsp_servers: Specifies the URL values for the OCSP Servers field.
        """
        if backend is not None:
            pulumi.set(__self__, "backend", backend)
        if crl_distribution_points is not None:
            pulumi.set(__self__, "crl_distribution_points", crl_distribution_points)
        if issuing_certificates is not None:
            pulumi.set(__self__, "issuing_certificates", issuing_certificates)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if ocsp_servers is not None:
            pulumi.set(__self__, "ocsp_servers", ocsp_servers)

    @property
    @pulumi.getter
    def backend(self) -> Optional[pulumi.Input[str]]:
        """
        The path the PKI secret backend is mounted at, with no leading or trailing `/`s.
        """
        return pulumi.get(self, "backend")

    @backend.setter
    def backend(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "backend", value)

    @property
    @pulumi.getter(name="crlDistributionPoints")
    def crl_distribution_points(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Specifies the URL values for the CRL Distribution Points field.
        """
        return pulumi.get(self, "crl_distribution_points")

    @crl_distribution_points.setter
    def crl_distribution_points(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "crl_distribution_points", value)

    @property
    @pulumi.getter(name="issuingCertificates")
    def issuing_certificates(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Specifies the URL values for the Issuing Certificate field.
        """
        return pulumi.get(self, "issuing_certificates")

    @issuing_certificates.setter
    def issuing_certificates(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "issuing_certificates", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        The namespace to provision the resource in.
        The value should not contain leading or trailing forward slashes.
        The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
        *Available only for Vault Enterprise*.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter(name="ocspServers")
    def ocsp_servers(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Specifies the URL values for the OCSP Servers field.
        """
        return pulumi.get(self, "ocsp_servers")

    @ocsp_servers.setter
    def ocsp_servers(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "ocsp_servers", value)


class SecretBackendConfigUrls(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 backend: Optional[pulumi.Input[str]] = None,
                 crl_distribution_points: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 issuing_certificates: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 ocsp_servers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 __props__=None):
        """
        Allows setting the issuing certificate endpoints, CRL distribution points, and OCSP server endpoints that will be encoded into issued certificates.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_vault as vault

        root = vault.Mount("root",
            path="pki-root",
            type="pki",
            description="root PKI",
            default_lease_ttl_seconds=8640000,
            max_lease_ttl_seconds=8640000)
        example = vault.pki_secret.SecretBackendConfigUrls("example",
            backend=root.path,
            issuing_certificates=["http://127.0.0.1:8200/v1/pki/ca"])
        ```

        ## Import

        The PKI config URLs can be imported using the resource's `id`. In the case of the example above the `id` would be `pki-root/config/urls`, where the `pki-root` component is the resource's `backend`, e.g.

        ```sh
         $ pulumi import vault:pkiSecret/secretBackendConfigUrls:SecretBackendConfigUrls example pki-root/config/urls
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] backend: The path the PKI secret backend is mounted at, with no leading or trailing `/`s.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] crl_distribution_points: Specifies the URL values for the CRL Distribution Points field.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] issuing_certificates: Specifies the URL values for the Issuing Certificate field.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] ocsp_servers: Specifies the URL values for the OCSP Servers field.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: SecretBackendConfigUrlsArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Allows setting the issuing certificate endpoints, CRL distribution points, and OCSP server endpoints that will be encoded into issued certificates.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_vault as vault

        root = vault.Mount("root",
            path="pki-root",
            type="pki",
            description="root PKI",
            default_lease_ttl_seconds=8640000,
            max_lease_ttl_seconds=8640000)
        example = vault.pki_secret.SecretBackendConfigUrls("example",
            backend=root.path,
            issuing_certificates=["http://127.0.0.1:8200/v1/pki/ca"])
        ```

        ## Import

        The PKI config URLs can be imported using the resource's `id`. In the case of the example above the `id` would be `pki-root/config/urls`, where the `pki-root` component is the resource's `backend`, e.g.

        ```sh
         $ pulumi import vault:pkiSecret/secretBackendConfigUrls:SecretBackendConfigUrls example pki-root/config/urls
        ```

        :param str resource_name: The name of the resource.
        :param SecretBackendConfigUrlsArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(SecretBackendConfigUrlsArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 backend: Optional[pulumi.Input[str]] = None,
                 crl_distribution_points: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 issuing_certificates: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 ocsp_servers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = SecretBackendConfigUrlsArgs.__new__(SecretBackendConfigUrlsArgs)

            if backend is None and not opts.urn:
                raise TypeError("Missing required property 'backend'")
            __props__.__dict__["backend"] = backend
            __props__.__dict__["crl_distribution_points"] = crl_distribution_points
            __props__.__dict__["issuing_certificates"] = issuing_certificates
            __props__.__dict__["namespace"] = namespace
            __props__.__dict__["ocsp_servers"] = ocsp_servers
        super(SecretBackendConfigUrls, __self__).__init__(
            'vault:pkiSecret/secretBackendConfigUrls:SecretBackendConfigUrls',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            backend: Optional[pulumi.Input[str]] = None,
            crl_distribution_points: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            issuing_certificates: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            namespace: Optional[pulumi.Input[str]] = None,
            ocsp_servers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None) -> 'SecretBackendConfigUrls':
        """
        Get an existing SecretBackendConfigUrls resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] backend: The path the PKI secret backend is mounted at, with no leading or trailing `/`s.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] crl_distribution_points: Specifies the URL values for the CRL Distribution Points field.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] issuing_certificates: Specifies the URL values for the Issuing Certificate field.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] ocsp_servers: Specifies the URL values for the OCSP Servers field.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _SecretBackendConfigUrlsState.__new__(_SecretBackendConfigUrlsState)

        __props__.__dict__["backend"] = backend
        __props__.__dict__["crl_distribution_points"] = crl_distribution_points
        __props__.__dict__["issuing_certificates"] = issuing_certificates
        __props__.__dict__["namespace"] = namespace
        __props__.__dict__["ocsp_servers"] = ocsp_servers
        return SecretBackendConfigUrls(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def backend(self) -> pulumi.Output[str]:
        """
        The path the PKI secret backend is mounted at, with no leading or trailing `/`s.
        """
        return pulumi.get(self, "backend")

    @property
    @pulumi.getter(name="crlDistributionPoints")
    def crl_distribution_points(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        Specifies the URL values for the CRL Distribution Points field.
        """
        return pulumi.get(self, "crl_distribution_points")

    @property
    @pulumi.getter(name="issuingCertificates")
    def issuing_certificates(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        Specifies the URL values for the Issuing Certificate field.
        """
        return pulumi.get(self, "issuing_certificates")

    @property
    @pulumi.getter
    def namespace(self) -> pulumi.Output[Optional[str]]:
        """
        The namespace to provision the resource in.
        The value should not contain leading or trailing forward slashes.
        The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
        *Available only for Vault Enterprise*.
        """
        return pulumi.get(self, "namespace")

    @property
    @pulumi.getter(name="ocspServers")
    def ocsp_servers(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        Specifies the URL values for the OCSP Servers field.
        """
        return pulumi.get(self, "ocsp_servers")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['SecretBackendDynamicRoleArgs', 'SecretBackendDynamicRole']

@pulumi.input_type
class SecretBackendDynamicRoleArgs:
    def __init__(__self__, *,
                 creation_ldif: pulumi.Input[str],
                 deletion_ldif: pulumi.Input[str],
                 role_name: pulumi.Input[str],
                 default_ttl: Optional[pulumi.Input[int]] = None,
                 max_ttl: Optional[pulumi.Input[int]] = None,
                 mount: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 rollback_ldif: Optional[pulumi.Input[str]] = None,
                 username_template: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a SecretBackendDynamicRole resource.
        :param pulumi.Input[str] creation_ldif: A templatized LDIF string used to create a user
               account. This may contain multiple LDIF entries. The `creation_ldif` can also
               be used to add the user account to an existing group. All LDIF entries are
               performed in order. If Vault encounters an error while executing the
               `creation_ldif` it will stop at the first error and not execute any remaining
               LDIF entries. If an error occurs and `rollback_ldif` is specified, the LDIF
               entries in `rollback_ldif` will be executed. See `rollback_ldif` for more
               details. This field may optionally be provided as a base64 encoded string.
        :param pulumi.Input[str] deletion_ldif: A templatized LDIF string used to delete the
               user account once its TTL has expired. This may contain multiple LDIF
               entries. All LDIF entries are performed in order. If Vault encounters an
               error while executing an entry in the `deletion_ldif` it will attempt to
               continue executing any remaining entries. This field may optionally be
               provided as a base64 encoded string.
        :param pulumi.Input[str] role_name: Name of the role.
        :param pulumi.Input[int] default_ttl: Specifies the TTL for the leases associated with this role.
        :param pulumi.Input[int] max_ttl: Specifies the maximum TTL for the leases associated with this role.
        :param pulumi.Input[str] mount: The unique path this backend should be mounted at. Must
               not begin or end with a `/`. Defaults to `ldap`.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[str] rollback_ldif: A templatized LDIF string used to attempt to
               rollback any changes in the event that execution of the `creation_ldif` results
               in an error. This may contain multiple LDIF entries. All LDIF entries are
               performed in order. If Vault encounters an error while executing an entry in
               the `rollback_ldif` it will attempt to continue executing any remaining
               entries. This field may optionally be provided as a base64 encoded string.
        :param pulumi.Input[str] username_template: A template used to generate a dynamic
               username. This will be used to fill in the `.Username` field within the
               `creation_ldif` string.
        """
        pulumi.set(__self__, "creation_ldif", creation_ldif)
        pulumi.set(__self__, "deletion_ldif", deletion_ldif)
        pulumi.set(__self__, "role_name", role_name)
        if default_ttl is not None:
            pulumi.set(__self__, "default_ttl", default_ttl)
        if max_ttl is not None:
            pulumi.set(__self__, "max_ttl", max_ttl)
        if mount is not None:
            pulumi.set(__self__, "mount", mount)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if rollback_ldif is not None:
            pulumi.set(__self__, "rollback_ldif", rollback_ldif)
        if username_template is not None:
            pulumi.set(__self__, "username_template", username_template)

    @property
    @pulumi.getter(name="creationLdif")
    def creation_ldif(self) -> pulumi.Input[str]:
        """
        A templatized LDIF string used to create a user
        account. This may contain multiple LDIF entries. The `creation_ldif` can also
        be used to add the user account to an existing group. All LDIF entries are
        performed in order. If Vault encounters an error while executing the
        `creation_ldif` it will stop at the first error and not execute any remaining
        LDIF entries. If an error occurs and `rollback_ldif` is specified, the LDIF
        entries in `rollback_ldif` will be executed. See `rollback_ldif` for more
        details. This field may optionally be provided as a base64 encoded string.
        """
        return pulumi.get(self, "creation_ldif")

    @creation_ldif.setter
    def creation_ldif(self, value: pulumi.Input[str]):
        pulumi.set(self, "creation_ldif", value)

    @property
    @pulumi.getter(name="deletionLdif")
    def deletion_ldif(self) -> pulumi.Input[str]:
        """
        A templatized LDIF string used to delete the
        user account once its TTL has expired. This may contain multiple LDIF
        entries. All LDIF entries are performed in order. If Vault encounters an
        error while executing an entry in the `deletion_ldif` it will attempt to
        continue executing any remaining entries. This field may optionally be
        provided as a base64 encoded string.
        """
        return pulumi.get(self, "deletion_ldif")

    @deletion_ldif.setter
    def deletion_ldif(self, value: pulumi.Input[str]):
        pulumi.set(self, "deletion_ldif", value)

    @property
    @pulumi.getter(name="roleName")
    def role_name(self) -> pulumi.Input[str]:
        """
        Name of the role.
        """
        return pulumi.get(self, "role_name")

    @role_name.setter
    def role_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "role_name", value)

    @property
    @pulumi.getter(name="defaultTtl")
    def default_ttl(self) -> Optional[pulumi.Input[int]]:
        """
        Specifies the TTL for the leases associated with this role.
        """
        return pulumi.get(self, "default_ttl")

    @default_ttl.setter
    def default_ttl(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "default_ttl", value)

    @property
    @pulumi.getter(name="maxTtl")
    def max_ttl(self) -> Optional[pulumi.Input[int]]:
        """
        Specifies the maximum TTL for the leases associated with this role.
        """
        return pulumi.get(self, "max_ttl")

    @max_ttl.setter
    def max_ttl(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "max_ttl", value)

    @property
    @pulumi.getter
    def mount(self) -> Optional[pulumi.Input[str]]:
        """
        The unique path this backend should be mounted at. Must
        not begin or end with a `/`. Defaults to `ldap`.
        """
        return pulumi.get(self, "mount")

    @mount.setter
    def mount(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "mount", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        The namespace to provision the resource in.
        The value should not contain leading or trailing forward slashes.
        The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
        *Available only for Vault Enterprise*.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter(name="rollbackLdif")
    def rollback_ldif(self) -> Optional[pulumi.Input[str]]:
        """
        A templatized LDIF string used to attempt to
        rollback any changes in the event that execution of the `creation_ldif` results
        in an error. This may contain multiple LDIF entries. All LDIF entries are
        performed in order. If Vault encounters an error while executing an entry in
        the `rollback_ldif` it will attempt to continue executing any remaining
        entries. This field may optionally be provided as a base64 encoded string.
        """
        return pulumi.get(self, "rollback_ldif")

    @rollback_ldif.setter
    def rollback_ldif(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "rollback_ldif", value)

    @property
    @pulumi.getter(name="usernameTemplate")
    def username_template(self) -> Optional[pulumi.Input[str]]:
        """
        A template used to generate a dynamic
        username. This will be used to fill in the `.Username` field within the
        `creation_ldif` string.
        """
        return pulumi.get(self, "username_template")

    @username_template.setter
    def username_template(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "username_template", value)


@pulumi.input_type
class _SecretBackendDynamicRoleState:
    def __init__(__self__, *,
                 creation_ldif: Optional[pulumi.Input[str]] = None,
                 default_ttl: Optional[pulumi.Input[int]] = None,
                 deletion_ldif: Optional[pulumi.Input[str]] = None,
                 max_ttl: Optional[pulumi.Input[int]] = None,
                 mount: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 role_name: Optional[pulumi.Input[str]] = None,
                 rollback_ldif: Optional[pulumi.Input[str]] = None,
                 username_template: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering SecretBackendDynamicRole resources.
        :param pulumi.Input[str] creation_ldif: A templatized LDIF string used to create a user
               account. This may contain multiple LDIF entries. The `creation_ldif` can also
               be used to add the user account to an existing group. All LDIF entries are
               performed in order. If Vault encounters an error while executing the
               `creation_ldif` it will stop at the first error and not execute any remaining
               LDIF entries. If an error occurs and `rollback_ldif` is specified, the LDIF
               entries in `rollback_ldif` will be executed. See `rollback_ldif` for more
               details. This field may optionally be provided as a base64 encoded string.
        :param pulumi.Input[int] default_ttl: Specifies the TTL for the leases associated with this role.
        :param pulumi.Input[str] deletion_ldif: A templatized LDIF string used to delete the
               user account once its TTL has expired. This may contain multiple LDIF
               entries. All LDIF entries are performed in order. If Vault encounters an
               error while executing an entry in the `deletion_ldif` it will attempt to
               continue executing any remaining entries. This field may optionally be
               provided as a base64 encoded string.
        :param pulumi.Input[int] max_ttl: Specifies the maximum TTL for the leases associated with this role.
        :param pulumi.Input[str] mount: The unique path this backend should be mounted at. Must
               not begin or end with a `/`. Defaults to `ldap`.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[str] role_name: Name of the role.
        :param pulumi.Input[str] rollback_ldif: A templatized LDIF string used to attempt to
               rollback any changes in the event that execution of the `creation_ldif` results
               in an error. This may contain multiple LDIF entries. All LDIF entries are
               performed in order. If Vault encounters an error while executing an entry in
               the `rollback_ldif` it will attempt to continue executing any remaining
               entries. This field may optionally be provided as a base64 encoded string.
        :param pulumi.Input[str] username_template: A template used to generate a dynamic
               username. This will be used to fill in the `.Username` field within the
               `creation_ldif` string.
        """
        if creation_ldif is not None:
            pulumi.set(__self__, "creation_ldif", creation_ldif)
        if default_ttl is not None:
            pulumi.set(__self__, "default_ttl", default_ttl)
        if deletion_ldif is not None:
            pulumi.set(__self__, "deletion_ldif", deletion_ldif)
        if max_ttl is not None:
            pulumi.set(__self__, "max_ttl", max_ttl)
        if mount is not None:
            pulumi.set(__self__, "mount", mount)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if role_name is not None:
            pulumi.set(__self__, "role_name", role_name)
        if rollback_ldif is not None:
            pulumi.set(__self__, "rollback_ldif", rollback_ldif)
        if username_template is not None:
            pulumi.set(__self__, "username_template", username_template)

    @property
    @pulumi.getter(name="creationLdif")
    def creation_ldif(self) -> Optional[pulumi.Input[str]]:
        """
        A templatized LDIF string used to create a user
        account. This may contain multiple LDIF entries. The `creation_ldif` can also
        be used to add the user account to an existing group. All LDIF entries are
        performed in order. If Vault encounters an error while executing the
        `creation_ldif` it will stop at the first error and not execute any remaining
        LDIF entries. If an error occurs and `rollback_ldif` is specified, the LDIF
        entries in `rollback_ldif` will be executed. See `rollback_ldif` for more
        details. This field may optionally be provided as a base64 encoded string.
        """
        return pulumi.get(self, "creation_ldif")

    @creation_ldif.setter
    def creation_ldif(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "creation_ldif", value)

    @property
    @pulumi.getter(name="defaultTtl")
    def default_ttl(self) -> Optional[pulumi.Input[int]]:
        """
        Specifies the TTL for the leases associated with this role.
        """
        return pulumi.get(self, "default_ttl")

    @default_ttl.setter
    def default_ttl(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "default_ttl", value)

    @property
    @pulumi.getter(name="deletionLdif")
    def deletion_ldif(self) -> Optional[pulumi.Input[str]]:
        """
        A templatized LDIF string used to delete the
        user account once its TTL has expired. This may contain multiple LDIF
        entries. All LDIF entries are performed in order. If Vault encounters an
        error while executing an entry in the `deletion_ldif` it will attempt to
        continue executing any remaining entries. This field may optionally be
        provided as a base64 encoded string.
        """
        return pulumi.get(self, "deletion_ldif")

    @deletion_ldif.setter
    def deletion_ldif(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "deletion_ldif", value)

    @property
    @pulumi.getter(name="maxTtl")
    def max_ttl(self) -> Optional[pulumi.Input[int]]:
        """
        Specifies the maximum TTL for the leases associated with this role.
        """
        return pulumi.get(self, "max_ttl")

    @max_ttl.setter
    def max_ttl(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "max_ttl", value)

    @property
    @pulumi.getter
    def mount(self) -> Optional[pulumi.Input[str]]:
        """
        The unique path this backend should be mounted at. Must
        not begin or end with a `/`. Defaults to `ldap`.
        """
        return pulumi.get(self, "mount")

    @mount.setter
    def mount(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "mount", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        The namespace to provision the resource in.
        The value should not contain leading or trailing forward slashes.
        The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
        *Available only for Vault Enterprise*.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter(name="roleName")
    def role_name(self) -> Optional[pulumi.Input[str]]:
        """
        Name of the role.
        """
        return pulumi.get(self, "role_name")

    @role_name.setter
    def role_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "role_name", value)

    @property
    @pulumi.getter(name="rollbackLdif")
    def rollback_ldif(self) -> Optional[pulumi.Input[str]]:
        """
        A templatized LDIF string used to attempt to
        rollback any changes in the event that execution of the `creation_ldif` results
        in an error. This may contain multiple LDIF entries. All LDIF entries are
        performed in order. If Vault encounters an error while executing an entry in
        the `rollback_ldif` it will attempt to continue executing any remaining
        entries. This field may optionally be provided as a base64 encoded string.
        """
        return pulumi.get(self, "rollback_ldif")

    @rollback_ldif.setter
    def rollback_ldif(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "rollback_ldif", value)

    @property
    @pulumi.getter(name="usernameTemplate")
    def username_template(self) -> Optional[pulumi.Input[str]]:
        """
        A template used to generate a dynamic
        username. This will be used to fill in the `.Username` field within the
        `creation_ldif` string.
        """
        return pulumi.get(self, "username_template")

    @username_template.setter
    def username_template(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "username_template", value)


class SecretBackendDynamicRole(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 creation_ldif: Optional[pulumi.Input[str]] = None,
                 default_ttl: Optional[pulumi.Input[int]] = None,
                 deletion_ldif: Optional[pulumi.Input[str]] = None,
                 max_ttl: Optional[pulumi.Input[int]] = None,
                 mount: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 role_name: Optional[pulumi.Input[str]] = None,
                 rollback_ldif: Optional[pulumi.Input[str]] = None,
                 username_template: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_vault as vault

        config = vault.ldap.SecretBackend("config",
            path="my-custom-ldap",
            binddn="CN=Administrator,CN=Users,DC=corp,DC=example,DC=net",
            bindpass="SuperSecretPassw0rd",
            url="ldaps://localhost",
            userdn="CN=Users,DC=corp,DC=example,DC=net")
        role = vault.ldap.SecretBackendDynamicRole("role",
            mount=config.path,
            role_name="alice",
            creation_ldif=\"\"\"dn: cn={{.Username}},ou=users,dc=learn,dc=example
        objectClass: person
        objectClass: top
        cn: learn
        sn: {{.Password | utf16le | base64}}
        memberOf: cn=dev,ou=groups,dc=learn,dc=example
        userPassword: {{.Password}}
        \"\"\",
            deletion_ldif=\"\"\"dn: cn={{.Username}},ou=users,dc=learn,dc=example
        changetype: delete
          rollback_ldif = <<EOT
        dn: cn={{.Username}},ou=users,dc=learn,dc=example
        changetype: delete
        \"\"\")
        ```

        ## Import

        LDAP secret backend dynamic role can be imported using the full path to the role of the form`<mount_path>/dynamic-role/<role_name>` e.g.

        ```sh
         $ pulumi import vault:ldap/secretBackendDynamicRole:SecretBackendDynamicRole role ldap/role/dynamic-role
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] creation_ldif: A templatized LDIF string used to create a user
               account. This may contain multiple LDIF entries. The `creation_ldif` can also
               be used to add the user account to an existing group. All LDIF entries are
               performed in order. If Vault encounters an error while executing the
               `creation_ldif` it will stop at the first error and not execute any remaining
               LDIF entries. If an error occurs and `rollback_ldif` is specified, the LDIF
               entries in `rollback_ldif` will be executed. See `rollback_ldif` for more
               details. This field may optionally be provided as a base64 encoded string.
        :param pulumi.Input[int] default_ttl: Specifies the TTL for the leases associated with this role.
        :param pulumi.Input[str] deletion_ldif: A templatized LDIF string used to delete the
               user account once its TTL has expired. This may contain multiple LDIF
               entries. All LDIF entries are performed in order. If Vault encounters an
               error while executing an entry in the `deletion_ldif` it will attempt to
               continue executing any remaining entries. This field may optionally be
               provided as a base64 encoded string.
        :param pulumi.Input[int] max_ttl: Specifies the maximum TTL for the leases associated with this role.
        :param pulumi.Input[str] mount: The unique path this backend should be mounted at. Must
               not begin or end with a `/`. Defaults to `ldap`.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[str] role_name: Name of the role.
        :param pulumi.Input[str] rollback_ldif: A templatized LDIF string used to attempt to
               rollback any changes in the event that execution of the `creation_ldif` results
               in an error. This may contain multiple LDIF entries. All LDIF entries are
               performed in order. If Vault encounters an error while executing an entry in
               the `rollback_ldif` it will attempt to continue executing any remaining
               entries. This field may optionally be provided as a base64 encoded string.
        :param pulumi.Input[str] username_template: A template used to generate a dynamic
               username. This will be used to fill in the `.Username` field within the
               `creation_ldif` string.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: SecretBackendDynamicRoleArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_vault as vault

        config = vault.ldap.SecretBackend("config",
            path="my-custom-ldap",
            binddn="CN=Administrator,CN=Users,DC=corp,DC=example,DC=net",
            bindpass="SuperSecretPassw0rd",
            url="ldaps://localhost",
            userdn="CN=Users,DC=corp,DC=example,DC=net")
        role = vault.ldap.SecretBackendDynamicRole("role",
            mount=config.path,
            role_name="alice",
            creation_ldif=\"\"\"dn: cn={{.Username}},ou=users,dc=learn,dc=example
        objectClass: person
        objectClass: top
        cn: learn
        sn: {{.Password | utf16le | base64}}
        memberOf: cn=dev,ou=groups,dc=learn,dc=example
        userPassword: {{.Password}}
        \"\"\",
            deletion_ldif=\"\"\"dn: cn={{.Username}},ou=users,dc=learn,dc=example
        changetype: delete
          rollback_ldif = <<EOT
        dn: cn={{.Username}},ou=users,dc=learn,dc=example
        changetype: delete
        \"\"\")
        ```

        ## Import

        LDAP secret backend dynamic role can be imported using the full path to the role of the form`<mount_path>/dynamic-role/<role_name>` e.g.

        ```sh
         $ pulumi import vault:ldap/secretBackendDynamicRole:SecretBackendDynamicRole role ldap/role/dynamic-role
        ```

        :param str resource_name: The name of the resource.
        :param SecretBackendDynamicRoleArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(SecretBackendDynamicRoleArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 creation_ldif: Optional[pulumi.Input[str]] = None,
                 default_ttl: Optional[pulumi.Input[int]] = None,
                 deletion_ldif: Optional[pulumi.Input[str]] = None,
                 max_ttl: Optional[pulumi.Input[int]] = None,
                 mount: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 role_name: Optional[pulumi.Input[str]] = None,
                 rollback_ldif: Optional[pulumi.Input[str]] = None,
                 username_template: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = SecretBackendDynamicRoleArgs.__new__(SecretBackendDynamicRoleArgs)

            if creation_ldif is None and not opts.urn:
                raise TypeError("Missing required property 'creation_ldif'")
            __props__.__dict__["creation_ldif"] = creation_ldif
            __props__.__dict__["default_ttl"] = default_ttl
            if deletion_ldif is None and not opts.urn:
                raise TypeError("Missing required property 'deletion_ldif'")
            __props__.__dict__["deletion_ldif"] = deletion_ldif
            __props__.__dict__["max_ttl"] = max_ttl
            __props__.__dict__["mount"] = mount
            __props__.__dict__["namespace"] = namespace
            if role_name is None and not opts.urn:
                raise TypeError("Missing required property 'role_name'")
            __props__.__dict__["role_name"] = role_name
            __props__.__dict__["rollback_ldif"] = rollback_ldif
            __props__.__dict__["username_template"] = username_template
        super(SecretBackendDynamicRole, __self__).__init__(
            'vault:ldap/secretBackendDynamicRole:SecretBackendDynamicRole',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            creation_ldif: Optional[pulumi.Input[str]] = None,
            default_ttl: Optional[pulumi.Input[int]] = None,
            deletion_ldif: Optional[pulumi.Input[str]] = None,
            max_ttl: Optional[pulumi.Input[int]] = None,
            mount: Optional[pulumi.Input[str]] = None,
            namespace: Optional[pulumi.Input[str]] = None,
            role_name: Optional[pulumi.Input[str]] = None,
            rollback_ldif: Optional[pulumi.Input[str]] = None,
            username_template: Optional[pulumi.Input[str]] = None) -> 'SecretBackendDynamicRole':
        """
        Get an existing SecretBackendDynamicRole resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] creation_ldif: A templatized LDIF string used to create a user
               account. This may contain multiple LDIF entries. The `creation_ldif` can also
               be used to add the user account to an existing group. All LDIF entries are
               performed in order. If Vault encounters an error while executing the
               `creation_ldif` it will stop at the first error and not execute any remaining
               LDIF entries. If an error occurs and `rollback_ldif` is specified, the LDIF
               entries in `rollback_ldif` will be executed. See `rollback_ldif` for more
               details. This field may optionally be provided as a base64 encoded string.
        :param pulumi.Input[int] default_ttl: Specifies the TTL for the leases associated with this role.
        :param pulumi.Input[str] deletion_ldif: A templatized LDIF string used to delete the
               user account once its TTL has expired. This may contain multiple LDIF
               entries. All LDIF entries are performed in order. If Vault encounters an
               error while executing an entry in the `deletion_ldif` it will attempt to
               continue executing any remaining entries. This field may optionally be
               provided as a base64 encoded string.
        :param pulumi.Input[int] max_ttl: Specifies the maximum TTL for the leases associated with this role.
        :param pulumi.Input[str] mount: The unique path this backend should be mounted at. Must
               not begin or end with a `/`. Defaults to `ldap`.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[str] role_name: Name of the role.
        :param pulumi.Input[str] rollback_ldif: A templatized LDIF string used to attempt to
               rollback any changes in the event that execution of the `creation_ldif` results
               in an error. This may contain multiple LDIF entries. All LDIF entries are
               performed in order. If Vault encounters an error while executing an entry in
               the `rollback_ldif` it will attempt to continue executing any remaining
               entries. This field may optionally be provided as a base64 encoded string.
        :param pulumi.Input[str] username_template: A template used to generate a dynamic
               username. This will be used to fill in the `.Username` field within the
               `creation_ldif` string.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _SecretBackendDynamicRoleState.__new__(_SecretBackendDynamicRoleState)

        __props__.__dict__["creation_ldif"] = creation_ldif
        __props__.__dict__["default_ttl"] = default_ttl
        __props__.__dict__["deletion_ldif"] = deletion_ldif
        __props__.__dict__["max_ttl"] = max_ttl
        __props__.__dict__["mount"] = mount
        __props__.__dict__["namespace"] = namespace
        __props__.__dict__["role_name"] = role_name
        __props__.__dict__["rollback_ldif"] = rollback_ldif
        __props__.__dict__["username_template"] = username_template
        return SecretBackendDynamicRole(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="creationLdif")
    def creation_ldif(self) -> pulumi.Output[str]:
        """
        A templatized LDIF string used to create a user
        account. This may contain multiple LDIF entries. The `creation_ldif` can also
        be used to add the user account to an existing group. All LDIF entries are
        performed in order. If Vault encounters an error while executing the
        `creation_ldif` it will stop at the first error and not execute any remaining
        LDIF entries. If an error occurs and `rollback_ldif` is specified, the LDIF
        entries in `rollback_ldif` will be executed. See `rollback_ldif` for more
        details. This field may optionally be provided as a base64 encoded string.
        """
        return pulumi.get(self, "creation_ldif")

    @property
    @pulumi.getter(name="defaultTtl")
    def default_ttl(self) -> pulumi.Output[Optional[int]]:
        """
        Specifies the TTL for the leases associated with this role.
        """
        return pulumi.get(self, "default_ttl")

    @property
    @pulumi.getter(name="deletionLdif")
    def deletion_ldif(self) -> pulumi.Output[str]:
        """
        A templatized LDIF string used to delete the
        user account once its TTL has expired. This may contain multiple LDIF
        entries. All LDIF entries are performed in order. If Vault encounters an
        error while executing an entry in the `deletion_ldif` it will attempt to
        continue executing any remaining entries. This field may optionally be
        provided as a base64 encoded string.
        """
        return pulumi.get(self, "deletion_ldif")

    @property
    @pulumi.getter(name="maxTtl")
    def max_ttl(self) -> pulumi.Output[Optional[int]]:
        """
        Specifies the maximum TTL for the leases associated with this role.
        """
        return pulumi.get(self, "max_ttl")

    @property
    @pulumi.getter
    def mount(self) -> pulumi.Output[Optional[str]]:
        """
        The unique path this backend should be mounted at. Must
        not begin or end with a `/`. Defaults to `ldap`.
        """
        return pulumi.get(self, "mount")

    @property
    @pulumi.getter
    def namespace(self) -> pulumi.Output[Optional[str]]:
        """
        The namespace to provision the resource in.
        The value should not contain leading or trailing forward slashes.
        The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
        *Available only for Vault Enterprise*.
        """
        return pulumi.get(self, "namespace")

    @property
    @pulumi.getter(name="roleName")
    def role_name(self) -> pulumi.Output[str]:
        """
        Name of the role.
        """
        return pulumi.get(self, "role_name")

    @property
    @pulumi.getter(name="rollbackLdif")
    def rollback_ldif(self) -> pulumi.Output[Optional[str]]:
        """
        A templatized LDIF string used to attempt to
        rollback any changes in the event that execution of the `creation_ldif` results
        in an error. This may contain multiple LDIF entries. All LDIF entries are
        performed in order. If Vault encounters an error while executing an entry in
        the `rollback_ldif` it will attempt to continue executing any remaining
        entries. This field may optionally be provided as a base64 encoded string.
        """
        return pulumi.get(self, "rollback_ldif")

    @property
    @pulumi.getter(name="usernameTemplate")
    def username_template(self) -> pulumi.Output[Optional[str]]:
        """
        A template used to generate a dynamic
        username. This will be used to fill in the `.Username` field within the
        `creation_ldif` string.
        """
        return pulumi.get(self, "username_template")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['OidcKeyArgs', 'OidcKey']

@pulumi.input_type
class OidcKeyArgs:
    def __init__(__self__, *,
                 algorithm: Optional[pulumi.Input[str]] = None,
                 allowed_client_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 rotation_period: Optional[pulumi.Input[int]] = None,
                 verification_ttl: Optional[pulumi.Input[int]] = None):
        """
        The set of arguments for constructing a OidcKey resource.
        :param pulumi.Input[str] algorithm: Signing algorithm to use. Signing algorithm to use.
               Allowed values are: RS256 (default), RS384, RS512, ES256, ES384, ES512, EdDSA.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] allowed_client_ids: Array of role client ID allowed to use this key for signing. If
               empty, no roles are allowed. If `["*"]`, all roles are allowed.
        :param pulumi.Input[str] name: Name of the OIDC Key to create.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[int] rotation_period: How often to generate a new signing key in number of seconds
        :param pulumi.Input[int] verification_ttl: "Controls how long the public portion of a signing key will be
               available for verification after being rotated in seconds.
        """
        if algorithm is not None:
            pulumi.set(__self__, "algorithm", algorithm)
        if allowed_client_ids is not None:
            pulumi.set(__self__, "allowed_client_ids", allowed_client_ids)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if rotation_period is not None:
            pulumi.set(__self__, "rotation_period", rotation_period)
        if verification_ttl is not None:
            pulumi.set(__self__, "verification_ttl", verification_ttl)

    @property
    @pulumi.getter
    def algorithm(self) -> Optional[pulumi.Input[str]]:
        """
        Signing algorithm to use. Signing algorithm to use.
        Allowed values are: RS256 (default), RS384, RS512, ES256, ES384, ES512, EdDSA.
        """
        return pulumi.get(self, "algorithm")

    @algorithm.setter
    def algorithm(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "algorithm", value)

    @property
    @pulumi.getter(name="allowedClientIds")
    def allowed_client_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Array of role client ID allowed to use this key for signing. If
        empty, no roles are allowed. If `["*"]`, all roles are allowed.
        """
        return pulumi.get(self, "allowed_client_ids")

    @allowed_client_ids.setter
    def allowed_client_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "allowed_client_ids", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        Name of the OIDC Key to create.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        The namespace to provision the resource in.
        The value should not contain leading or trailing forward slashes.
        The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
        *Available only for Vault Enterprise*.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter(name="rotationPeriod")
    def rotation_period(self) -> Optional[pulumi.Input[int]]:
        """
        How often to generate a new signing key in number of seconds
        """
        return pulumi.get(self, "rotation_period")

    @rotation_period.setter
    def rotation_period(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "rotation_period", value)

    @property
    @pulumi.getter(name="verificationTtl")
    def verification_ttl(self) -> Optional[pulumi.Input[int]]:
        """
        "Controls how long the public portion of a signing key will be
        available for verification after being rotated in seconds.
        """
        return pulumi.get(self, "verification_ttl")

    @verification_ttl.setter
    def verification_ttl(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "verification_ttl", value)


@pulumi.input_type
class _OidcKeyState:
    def __init__(__self__, *,
                 algorithm: Optional[pulumi.Input[str]] = None,
                 allowed_client_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 rotation_period: Optional[pulumi.Input[int]] = None,
                 verification_ttl: Optional[pulumi.Input[int]] = None):
        """
        Input properties used for looking up and filtering OidcKey resources.
        :param pulumi.Input[str] algorithm: Signing algorithm to use. Signing algorithm to use.
               Allowed values are: RS256 (default), RS384, RS512, ES256, ES384, ES512, EdDSA.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] allowed_client_ids: Array of role client ID allowed to use this key for signing. If
               empty, no roles are allowed. If `["*"]`, all roles are allowed.
        :param pulumi.Input[str] name: Name of the OIDC Key to create.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[int] rotation_period: How often to generate a new signing key in number of seconds
        :param pulumi.Input[int] verification_ttl: "Controls how long the public portion of a signing key will be
               available for verification after being rotated in seconds.
        """
        if algorithm is not None:
            pulumi.set(__self__, "algorithm", algorithm)
        if allowed_client_ids is not None:
            pulumi.set(__self__, "allowed_client_ids", allowed_client_ids)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if rotation_period is not None:
            pulumi.set(__self__, "rotation_period", rotation_period)
        if verification_ttl is not None:
            pulumi.set(__self__, "verification_ttl", verification_ttl)

    @property
    @pulumi.getter
    def algorithm(self) -> Optional[pulumi.Input[str]]:
        """
        Signing algorithm to use. Signing algorithm to use.
        Allowed values are: RS256 (default), RS384, RS512, ES256, ES384, ES512, EdDSA.
        """
        return pulumi.get(self, "algorithm")

    @algorithm.setter
    def algorithm(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "algorithm", value)

    @property
    @pulumi.getter(name="allowedClientIds")
    def allowed_client_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Array of role client ID allowed to use this key for signing. If
        empty, no roles are allowed. If `["*"]`, all roles are allowed.
        """
        return pulumi.get(self, "allowed_client_ids")

    @allowed_client_ids.setter
    def allowed_client_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "allowed_client_ids", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        Name of the OIDC Key to create.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        The namespace to provision the resource in.
        The value should not contain leading or trailing forward slashes.
        The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
        *Available only for Vault Enterprise*.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter(name="rotationPeriod")
    def rotation_period(self) -> Optional[pulumi.Input[int]]:
        """
        How often to generate a new signing key in number of seconds
        """
        return pulumi.get(self, "rotation_period")

    @rotation_period.setter
    def rotation_period(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "rotation_period", value)

    @property
    @pulumi.getter(name="verificationTtl")
    def verification_ttl(self) -> Optional[pulumi.Input[int]]:
        """
        "Controls how long the public portion of a signing key will be
        available for verification after being rotated in seconds.
        """
        return pulumi.get(self, "verification_ttl")

    @verification_ttl.setter
    def verification_ttl(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "verification_ttl", value)


class OidcKey(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 algorithm: Optional[pulumi.Input[str]] = None,
                 allowed_client_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 rotation_period: Optional[pulumi.Input[int]] = None,
                 verification_ttl: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_vault as vault

        key = vault.identity.OidcKey("key", algorithm="RS256")
        role_oidc_role = vault.identity.OidcRole("roleOidcRole", key=key.name)
        role_oidc_key_allowed_client_id = vault.identity.OidcKeyAllowedClientID("roleOidcKeyAllowedClientID",
            key_name=key.name,
            allowed_client_id=role_oidc_role.client_id)
        ```

        ## Import

        The key can be imported with the key name, for example:

        ```sh
         $ pulumi import vault:identity/oidcKey:OidcKey key key
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] algorithm: Signing algorithm to use. Signing algorithm to use.
               Allowed values are: RS256 (default), RS384, RS512, ES256, ES384, ES512, EdDSA.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] allowed_client_ids: Array of role client ID allowed to use this key for signing. If
               empty, no roles are allowed. If `["*"]`, all roles are allowed.
        :param pulumi.Input[str] name: Name of the OIDC Key to create.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[int] rotation_period: How often to generate a new signing key in number of seconds
        :param pulumi.Input[int] verification_ttl: "Controls how long the public portion of a signing key will be
               available for verification after being rotated in seconds.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[OidcKeyArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_vault as vault

        key = vault.identity.OidcKey("key", algorithm="RS256")
        role_oidc_role = vault.identity.OidcRole("roleOidcRole", key=key.name)
        role_oidc_key_allowed_client_id = vault.identity.OidcKeyAllowedClientID("roleOidcKeyAllowedClientID",
            key_name=key.name,
            allowed_client_id=role_oidc_role.client_id)
        ```

        ## Import

        The key can be imported with the key name, for example:

        ```sh
         $ pulumi import vault:identity/oidcKey:OidcKey key key
        ```

        :param str resource_name: The name of the resource.
        :param OidcKeyArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(OidcKeyArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 algorithm: Optional[pulumi.Input[str]] = None,
                 allowed_client_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 rotation_period: Optional[pulumi.Input[int]] = None,
                 verification_ttl: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = OidcKeyArgs.__new__(OidcKeyArgs)

            __props__.__dict__["algorithm"] = algorithm
            __props__.__dict__["allowed_client_ids"] = allowed_client_ids
            __props__.__dict__["name"] = name
            __props__.__dict__["namespace"] = namespace
            __props__.__dict__["rotation_period"] = rotation_period
            __props__.__dict__["verification_ttl"] = verification_ttl
        super(OidcKey, __self__).__init__(
            'vault:identity/oidcKey:OidcKey',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            algorithm: Optional[pulumi.Input[str]] = None,
            allowed_client_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            name: Optional[pulumi.Input[str]] = None,
            namespace: Optional[pulumi.Input[str]] = None,
            rotation_period: Optional[pulumi.Input[int]] = None,
            verification_ttl: Optional[pulumi.Input[int]] = None) -> 'OidcKey':
        """
        Get an existing OidcKey resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] algorithm: Signing algorithm to use. Signing algorithm to use.
               Allowed values are: RS256 (default), RS384, RS512, ES256, ES384, ES512, EdDSA.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] allowed_client_ids: Array of role client ID allowed to use this key for signing. If
               empty, no roles are allowed. If `["*"]`, all roles are allowed.
        :param pulumi.Input[str] name: Name of the OIDC Key to create.
        :param pulumi.Input[str] namespace: The namespace to provision the resource in.
               The value should not contain leading or trailing forward slashes.
               The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
               *Available only for Vault Enterprise*.
        :param pulumi.Input[int] rotation_period: How often to generate a new signing key in number of seconds
        :param pulumi.Input[int] verification_ttl: "Controls how long the public portion of a signing key will be
               available for verification after being rotated in seconds.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _OidcKeyState.__new__(_OidcKeyState)

        __props__.__dict__["algorithm"] = algorithm
        __props__.__dict__["allowed_client_ids"] = allowed_client_ids
        __props__.__dict__["name"] = name
        __props__.__dict__["namespace"] = namespace
        __props__.__dict__["rotation_period"] = rotation_period
        __props__.__dict__["verification_ttl"] = verification_ttl
        return OidcKey(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def algorithm(self) -> pulumi.Output[Optional[str]]:
        """
        Signing algorithm to use. Signing algorithm to use.
        Allowed values are: RS256 (default), RS384, RS512, ES256, ES384, ES512, EdDSA.
        """
        return pulumi.get(self, "algorithm")

    @property
    @pulumi.getter(name="allowedClientIds")
    def allowed_client_ids(self) -> pulumi.Output[Sequence[str]]:
        """
        Array of role client ID allowed to use this key for signing. If
        empty, no roles are allowed. If `["*"]`, all roles are allowed.
        """
        return pulumi.get(self, "allowed_client_ids")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        Name of the OIDC Key to create.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def namespace(self) -> pulumi.Output[Optional[str]]:
        """
        The namespace to provision the resource in.
        The value should not contain leading or trailing forward slashes.
        The `namespace` is always relative to the provider's configured [namespace](https://www.terraform.io/docs/providers/vault#namespace).
        *Available only for Vault Enterprise*.
        """
        return pulumi.get(self, "namespace")

    @property
    @pulumi.getter(name="rotationPeriod")
    def rotation_period(self) -> pulumi.Output[Optional[int]]:
        """
        How often to generate a new signing key in number of seconds
        """
        return pulumi.get(self, "rotation_period")

    @property
    @pulumi.getter(name="verificationTtl")
    def verification_ttl(self) -> pulumi.Output[Optional[int]]:
        """
        "Controls how long the public portion of a signing key will be
        available for verification after being rotated in seconds.
        """
        return pulumi.get(self, "verification_ttl")

